--[[ -------------------------------------------------------
-- I, Thrae of Maelstrom (aka Matthew Carras), hereby deem this
-- addon and all its contents public domain.
--
-- When working, this addon lets the caster and party or raid members
-- know about who enabled that Crusader Aura, Aspect of the Pack,
-- and/or Aspect of the Cheetah (latter only for yourself).
--
-- First it whispers the offender after a short period of time and
-- politely asks they remove it. If it's still there by the same
-- caster after a much longer period of time, it rats them out to
-- the party, raid, or battlegrounds members.
--
-- It only rats out Crusader Aura in PvP, where it really does make a
-- difference. Of course it checks if you're mounted.
--
-- It does not rat out yourself for any of them. You just get a
-- message in your DEFAULT_CHAT_FRAME.
--
-- Cheers,
-- Thrae of Maelstrom
-------------------------------------------------------- --]]

local addonName = "DontDazeMeBro"

-- Play it forward.
local EventFrame, CheckForStupidAuras, _

-- Change the various delays below if you want to give more or less time to the offending caster.
local c_timeToRatOnAura = 30 -- tells everybody possible
local c_timeToWhisperOnAura = 5
local c_timeToWhisperOnPack = 2

-- This adds a little extra time for raids and parties where it's not as big a
-- deal as in BGs. This is the delay to tell everybody possible.
local c_timeToRatOnPackByChannel = {
    ["BATTLEGROUND"] = 6,
    ["RAID"] = 12,
    ["PARTY"] = 12,
    ["YELL"] = 30,
}

-- The default to complain is every 30 seconds.
local c_complaintDelay = 30
local c_whisperDelay = 1

-- Change these to "BLAHBLAH" if you don't want them tested.
local c_aspectOfTheCheetah = GetSpellInfo(5118)
local c_aspectOfThePack = GetSpellInfo(13159)
local c_crusaderAura = GetSpellInfo(32223)

local c_unknownName = "?UNKNOWN?"

-- Engage.
local lastWhisperTime, lastComplaintTime = 0, 0
local auraDetectionTime, packDetectionTime = 0, 0
local cheetahDetected = nil
local whispered, ratted

local throttleUpdate = 1
local OnUpdate
do
    local timeSinceLastUpdate = 0

    OnUpdate = function(self, elapsed)
        timeSinceLastUpdate = timeSinceLastUpdate + elapsed

        if timeSinceLastUpdate >= throttleUpdate then
            CheckForStupidAuras(math.floor(timeSinceLastUpdate))
            timeSinceLastUpdate = 0
        end
    end
end

CheckForStupidAuras = function(elapsed)
  local currentTime = time()

  local inBG = select(2, GetInstanceInfo()) == "pvp"
  local channel = (inBG and "BATTLEGROUND") or (GetNumRaidMembers() > 0 and "RAID") or (GetNumPartyMembers() > 0 and "PARTY") or "YELL"
  local inPVP = GetZonePVPInfo()
  inPVP = inPVP and (inPVP == "combat" or inPVP == "arena")

  local name, _, _, _, _, _, _, unitCaster = UnitAura('player', c_crusaderAura)
  if name and not IsMounted() then
    local uname = UnitName(unitCaster) or c_unknownName
    auraDetectionTime = auraDetectionTime + (elapsed or 1)
    if auraDetectionTime < c_timeToRatOnAura then
      if not EventFrame:GetScript("OnUpdate") then
        EventFrame:SetScript("OnUpdate", OnUpdate)
        throttleUpdate = c_timeToWhisperOnAura
      end
    if auraDetectionTime >= c_timeToWhisperOnAura and (not whispered or not whispered[uname]) and currentTime - lastWhisperTime > c_whisperDelay then
        lastWhisperTime = currentTime
        if not whispered then whispered = {} end
        whispered[uname] = true

        if UnitIsUnit(unitCaster, "player") then
          DEFAULT_CHAT_FRAME:AddMessage(string.format("*** %s detected I have %s enabled in combat without being mounted! ***", addonName, name))
        elseif uname ~= c_unknownName then
          SendChatMessage(string.format("%s detected you have %s still enabled, though I'm not mounted, and now in combat. Please switch to a more useful aura for the duration of the battle. Thanks :)",
                                        addonName, name),
                          "WHISPER", nil, uname)
        end

        throttleUpdate = c_timeToRatOnAura
      end
    elseif (inBG or inPVP) and (not ratted or not ratted[uname]) and currentTime - lastComplaintTime > c_complaintDelay then
        lastComplaintTime = currentTime
        if not ratted then ratted = {} end
        ratted[uname] = true

      SendChatMessage(string.format("%s detected %s is still enabled by %s while I'm not mounted and have even been in combat for %d seconds. Can we please get a different aura?",
                                    addonName, name, uname, auraDetectionTime),
                                    channel)

      throttleUpdate = c_complaintDelay
    end
  end

  name, _, _, _, _, _, _, unitCaster = UnitAura('player', c_aspectOfThePack)
  if name then
    local uname = UnitName(unitCaster) or c_unknownName
    packDetectionTime = packDetectionTime + (elapsed or 1)
    if packDetectionTime < c_timeToRatOnPackByChannel[channel] then
      if not EventFrame:GetScript("OnUpdate") then
        EventFrame:SetScript("OnUpdate", OnUpdate)
        throttleUpdate = c_timeToWhisperOnPack
      end
      if packDetectionTime >= c_timeToWhisperOnPack and (not whispered or not whispered[uname]) and currentTime - lastWhisperTime > c_whisperDelay then
        lastWhisperTime = currentTime
        if not whispered then whispered = {} end
        whispered[uname] = true

        throttleUpdate = c_timeToRatOnPackByChannel[channel]

        if UnitIsUnit(unitCaster, "player") then
          DEFAULT_CHAT_FRAME:AddMessage(string.format("*** %s detected I have %s still enabled in combat! ***", addonName, name))
          return
        elseif uname ~= c_unknownName then
          SendChatMessage(string.format("%s detected you have %s still active and I'm in combat! Please turn it off. Thanks.",
                                        addonName, name),
                          "WHISPER", nil, uname)
          return
        end
      end
    elseif (not ratted or not ratted[uname]) and currentTime - lastComplaintTime > c_complaintDelay then
      lastComplaintTime = currentTime
      if not ratted then ratted = {} end
      ratted[uname] = true

      SendChatMessage(string.format("%s detected %s is still enabled by %s while we've been in combat for %d seconds.",
                                    addonName, name, uname, packDetectionTime),
                      channel)

      throttleUpdate = c_complaintDelay
      return
    end
  end

  name = UnitAura('player', c_aspectOfTheCheetah)
  if name and not cheetahDetected then
    DEFAULT_CHAT_FRAME:AddMessage(string.format("*** %s detected I have %s still enabled in combat! ***", addonName, name))
    cheetahDetected = true
  end
end

local function OnEvent(self, event)
  if event == "PLAYER_REGEN_ENABLED" then
    whispered, ratted = nil, nil
    cheetahDetected = nil
    lastComplaintTime, lastWhisperTime = 0,0
    auraDetectionTime, packDetectionTime = 0,0
    self:SetScript("OnUpdate", nil)
  elseif event == "PLAYER_REGEN_DISABLED" then
    CheckForStupidAuras()
  end
end
EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
EventFrame:SetScript("OnEvent", OnEvent)
